#!/usr/bin/ruby

@base_dir = Dir::getwd + "/" + File.dirname($0)

@module_name = "bag_of_keypoints_module"

@tarballs = [
  "BagOfKeypoint",
  "BagOfKeypointsModule",
  "IplImageOutModule",
  "IplImageViewerModule",
  "VideoCaptureModule",
  "ViewBokResult",
  "image"
]

@other_files = [
  "README.txt", 
  "Makefile", 
  "build.rb"
]


def exec(cmd)
  puts "<execute> #{cmd}"
  system(cmd)
end

def extract
  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    exec("tar xvfz #{f}-*tar.gz")
  }
end

def _recursive_exec(cmd, d)
  Dir::chdir(d)

  Dir.glob("*") {|f|
    _recursive_exec(cmd, f) if File::ftype(f) == "directory"
  }

  if File.exist? "Makefile"
    exec(cmd)
  end

  Dir::chdir("..")
end

def make
  @tarballs.each {|d|
    Dir::chdir(@base_dir)
    _recursive_exec("make", d)
  }
end

def clean
  @tarballs.each {|d|
    Dir::chdir(@base_dir)
    _recursive_exec("make clean", d)
  }
end

def tarball
  clean

  datestr = Time.now.strftime("%Y%m%d")

  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    Dir::chdir(f)
    if File.exist? "Makefile"
      system("make clean")
    end

    Dir::chdir(@base_dir)

    cmd = "tar cvfz #{f}-#{datestr}.tar.gz --exclude=.svn #{f}"
    puts cmd
    system(cmd)

  }

  Dir::chdir(@base_dir)
  tmpdir = "/tmp/#{@module_name}-#{datestr}"
  exec "mkdir -p #{tmpdir}"
  exec "mv *.tar.gz #{tmpdir}"
  @other_files.each {|f|
    exec "cp #{f} #{tmpdir}"
  }

  Dir::chdir(tmpdir)
  Dir::chdir("..")
  exec "tar cvfz #{@module_name}-#{datestr}.tar.gz #{@module_name}-#{datestr}" 
  exec "mv #{@module_name}-#{datestr}.tar.gz ~"
  exec "rm -rf #{tmpdir}"
end

def usage
  puts "usage : build.sh {extract|make|run|tarball|clean}"
  exit 0
end
 
if __FILE__ == $0
  usage if ARGV.size != 1

  puts "execute cmd=#{ARGV[0]}"
  puts "base_dir = #{@base_dir}"
  Dir::chdir(@base_dir)

  send(ARGV[0])
end
